CREATE SEQUENCE ID_SEQ;

CREATE TABLE TBLLOGINS (
       LOGINID                NUMBER(10) NOT NULL
                                   CHECK ("LOGINID" IS NOT NULL),
       EMAIL                VARCHAR2(100) NOT NULL
                                   CHECK ("EMAIL" IS NOT NULL),
       PASSWORD             VARCHAR2(50) NULL
);

CREATE UNIQUE INDEX PK_TBLLOGINS ON TBLLOGINS
(
       LOGINID
);

CREATE UNIQUE INDEX EMAIL ON TBLLOGINS
(
       EMAIL
);

ALTER TABLE TBLLOGINS
       ADD  ( PRIMARY KEY (LOGINID) ) ;


CREATE TABLE TBLMSGS (
       MSGID                NUMBER(10) NOT NULL
                                   CHECK ("MSGID" IS NOT NULL),
       MSG                  LONG NULL
);

CREATE UNIQUE INDEX PK_TBLMSGS ON TBLMSGS
(
       MSGID
);


ALTER TABLE TBLMSGS
       ADD  ( PRIMARY KEY (MSGID) ) ;


CREATE TABLE TBLQUARANTINE (
       QUARID               NUMBER(10) NOT NULL
                                   CHECK (("QUARID" IS NOT NULL)),
       EMAILFROM            VARCHAR2(100) NULL,
       EMAILTO              VARCHAR2(100) NULL,
       SUBJECT              VARCHAR2(100) NULL,
       MSGDATE              DATE NULL,
       MSGID                NUMBER(10) NULL,
       REJECTID             NUMBER(10) NULL,
       REJECTDETAILS        VARCHAR2(200) NULL,
       DELIVER              NUMBER(2) NULL,
       EXPIRE               NUMBER(2) NULL,
       SERVERID             NUMBER(10) NULL
);

CREATE TABLE TBLSERVERS (
  SERVERID NUMBER(10) NOT NULL,
  SERVERNAME VARCHAR2(50) NOT NULL
);

CREATE UNIQUE INDEX QUARID ON TBLQUARANTINE
(
       QUARID
);

CREATE INDEX EMAILFROM ON TBLQUARANTINE
(
       EMAILFROM
);

CREATE INDEX EMAILTO ON TBLQUARANTINE
(
       EMAILTO
);

CREATE INDEX MSGDATE ON TBLQUARANTINE
(
       MSGDATE
);

CREATE INDEX MSGID ON TBLQUARANTINE
(
       MSGID
);

CREATE INDEX REJECTID ON TBLQUARANTINE
(
       REJECTID
);

CREATE INDEX SUBJECT ON TBLQUARANTINE
(
       SUBJECT
);


ALTER TABLE TBLQUARANTINE
       ADD  ( PRIMARY KEY (QUARID) ) ;


CREATE TABLE TBLREJECTCODES (
       REJECTID             NUMBER(10) NOT NULL
                                   CHECK ("REJECTID" IS NOT NULL),
       REJECTDESC           VARCHAR2(100) NULL
);


ALTER TABLE TBLQUARANTINE
       ADD  ( FOREIGN KEY (MSGID)
                             REFERENCES TBLMSGS
                             ON DELETE CASCADE ) ;




create trigger TBLMSGS_AUTOINCREMENT BEFORE INSERT on TBLMSGS for each row
DECLARE l_pkval number(22)\;   
BEGIN    
  select ID_SEQ.nextval into l_pkval from dual\;   
  :new.MSGID :=  l_pkval\;   
END\;
;

create trigger tD_TBLMSGS after DELETE on TBLMSGS for each row
declare numrows INTEGER\;
begin
    delete from TBLQUARANTINE where TBLQUARANTINE.MSGID = :old.MSGID\;
end\;
;


create trigger tU_TBLMSGS after UPDATE on TBLMSGS for each row
declare numrows INTEGER\;
begin
  if :old.MSGID <> :new.MSGID
  then
    select count(*) into numrows from TBLQUARANTINE where TBLQUARANTINE.MSGID = :old.MSGID\;
    if (numrows > 0)
    then 
      raise_application_error(
        -20005,
        'Cannot UPDATE "TBLMSGS" because "TBLQUARANTINE" exists.'
      )\;
    end if\;
  end if\;
end\;
;

create trigger TBLQUARANTINE_AUTOINCREMENT BEFORE INSERT on TBLQUARANTINE for each row
DECLARE l_pkval number(22)\;    
BEGIN  
  select ID_SEQ.nextval into l_pkval from dual\;    
  :new.QUARID :=  l_pkval\;    
END\;
;


create trigger TBLLOGINS_AUTOINCREMENT BEFORE INSERT on TBLLOGINS for each row
DECLARE l_pkval number(22)\;    
BEGIN  
  select ID_SEQ.nextval into l_pkval from dual\;    
  :new.LOGINID :=  l_pkval\;    
END\;
;

create trigger TBLSERVERS_AUTOINCREMENT BEFORE INSERT on TBLSERVERS for each row
DECLARE l_pkval number(22)\;    
BEGIN  
  select ID_SEQ.nextval into l_pkval from dual\;    
  :new.SERVERID :=  l_pkval\;    
END\;
;